/*
 * Copyright 2007-2009 Florian Köberle
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.delphigl.files.ftp;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.SocketException;
import java.nio.charset.Charset;

import org.apache.commons.net.ftp.FTP;
import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.net.ftp.FTPFile;

public class SimpleFTPClient {
	private final String host;
	private final String directoryPrefix;
	private final String username;
	private final String password;
	private final FTPClient ftp;

	public SimpleFTPClient(String host, String directoryPrefix,
			String username, String password) throws IOException {
		this.host = host;
		this.directoryPrefix = directoryPrefix;
		this.username = username;
		this.password = password;
		this.ftp = new FTPClient();
	}

	public void connect() throws SocketException, IOException {
		ftp.connect(host);
		ftp.setFileType(FTPClient.BINARY_FILE_TYPE);
		boolean loginSuccessful = false;
		try {
			loginSuccessful = ftp.login(username, password);
			if (!loginSuccessful) {
				throw new IOException("ftp login failed");
			}
			/*
			 * The file type defaults to text file. When you transfer the file
			 * as text file, it can happen that a few bytes do not get
			 * transfered.
			 */
			if (!ftp.setFileType(FTP.BINARY_FILE_TYPE)) {
				throw new IOException(
						"upload failed, as content type can't be set to binary.");
			}
		} finally {
			if (!loginSuccessful) {
				ftp.disconnect();
			}
		}
	}

	public void disconnect() throws IOException {
		if (ftp.isConnected()) {
			try {
				ftp.logout();
			} finally {
				ftp.disconnect();
			}
		}
	}

	/*
	 * public void downloadFile(String remotePath, File localFile) throws
	 * IOException { remotePath = directoryPrefix + "/" + remotePath;
	 * localFile.delete(); FTPFile[] filesWithThatName =
	 * ftp.listFiles(remotePath); if (filesWithThatName.length != 1) { throw new
	 * FileNotFoundException("Unable to find remote file"); } final
	 * FileOutputStream fileOutputStream = new FileOutputStream(localFile); try
	 * { final BufferedOutputStream outputStream = new
	 * BufferedOutputStream(fileOutputStream); try { boolean success =
	 * ftp.retrieveFile(remotePath, outputStream); if (!success) { throw new
	 * IOException("Unable to retrieve file"); } } finally {
	 * outputStream.close(); } } finally { fileOutputStream.close(); } }
	 */

	public void uploadFile(String remotePath, File localFile)
			throws IOException, FileNotFoundException {
		remotePath = directoryPrefix + "/" + remotePath;
		FileInputStream fileInputStream = new FileInputStream(localFile);
		try {
			BufferedInputStream bufferedInputStream = new BufferedInputStream(
					fileInputStream);
			try {
				boolean success = ftp
						.storeFile(remotePath, bufferedInputStream);
				if (!success) {
					throw new IOException("upload failed");
				}
			} finally {

			}
		} finally {
			fileInputStream.close();
		}
	}

	public void deleteFile(String remotePath) throws IOException {
		remotePath = directoryPrefix + remotePath;
		ftp.deleteFile(remotePath);
	}

	/**
	 * @param remotePath
	 * @return
	 * @throws IOException
	 */
	public String downloadAsUTF8String(String remotePath) throws IOException {
		remotePath = directoryPrefix + "/" + remotePath;
		if (!fileExists(remotePath)) {
			throw new FileNotFoundException("Unable to find remote file");
		}
		final ByteArrayOutputStream outputStream = new ByteArrayOutputStream();

		final BufferedOutputStream bufferedOutputStream = new BufferedOutputStream(
				outputStream);
		try {
			boolean success = ftp
					.retrieveFile(remotePath, bufferedOutputStream);
			if (!success) {
				throw new IOException("Unable to retrieve file");
			}
		} finally {
			bufferedOutputStream.close();
		}
		return new String(outputStream.toByteArray(), Charset.forName("UTF-8"));
	}

	private boolean fileExists(String remotePath) throws IOException {
		final FTPFile[] filesWithThatName = ftp.listFiles(remotePath);
		return (filesWithThatName.length == 1);
	}
}
