/*
 * Copyright 2007-2009 Florian Köberle
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.delphigl.files.hashcalculation;

import static java.util.Arrays.sort;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.Comparator;

public class RecursiveHashCalculatorImplementation implements
		RecursiveHashCalculator {

	private static final class DirectoryFirstComperator implements
			Comparator<File> {
		@Override
		public int compare(File o1, File o2) {
			if (o1.isDirectory() && o2.isFile()) {
				return -1;
			}
			if (o1.isFile() && o2.isDirectory()) {
				return 1;
			}
			return o1.compareTo(o2);
		}
	}

	private final MasterHashCalculator hashCalculator;
	static final Comparator<File> FILE_COMPERATOR = new DirectoryFirstComperator();

	public RecursiveHashCalculatorImplementation() {
		hashCalculator = new MasterHashCalculator();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seecom.delphigl.files.hashcalculation.RecurSiveHashCalculatorInterface#
	 * calculateHashesOfDirectory(java.io.File,
	 * com.delphigl.files.hashcalculation.HashCalculationListener)
	 */
	public void calculateHashesOfDirectory(File directory,
			HashCalculationListener listener) throws IOException {
		if (!directory.isDirectory()) {
			throw new IllegalArgumentException(directory
					+ " is not a directory");
		}
		handleDirectory(directory, listener, 0, 1);
	}

	protected void handlePath(File path, HashCalculationListener listener,
			double startProgress, double progressSize) throws IOException {
		if (path.isDirectory()) {
			handleDirectory(path, listener, startProgress, progressSize);
		} else if (path.isFile()) {
			handleFile(path, listener, startProgress, progressSize);
		}
	}

	private void handleDirectory(File directory,
			HashCalculationListener listener, double startProgress,
			double progressSize) throws IOException {
		final File[] files = directory.listFiles();
		sort(files, FILE_COMPERATOR);
		final double entryProgressSize = progressSize / files.length;

		for (int i = 0; i < files.length; i++) {
			handlePath(files[i], listener, startProgress + i
					* entryProgressSize, entryProgressSize);
		}
	}

	private void handleFile(File file, HashCalculationListener listener,
			double startProgress, double progressSize) throws IOException {
		final InputStream inputStream = new FileInputStream(file);
		try {
			byte[] hashByteArray = hashCalculator.calculateHash(file.getName(),
					inputStream);
			listener.hashCalculated(file, byteArrayToHexString(hashByteArray));
			listener.handleProgress(startProgress + progressSize);
		} finally {
			inputStream.close();
		}
	}

	private static String byteArrayToHexString(byte[] array) {
		final StringWriter stringWriter = new StringWriter();
		final PrintWriter printer = new PrintWriter(stringWriter);
		for (byte b : array) {
			printer.printf("%x", b);
		}
		return stringWriter.toString();
	}

}
