/*
 * Copyright 2007-2009 Florian Köberle
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.delphigl.files.upload.hashcalculation;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Arrays;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import org.junit.Test;

import com.delphigl.files.hashcalculation.MasterHashCalculator;

public class MasterHashCalculatorTest {

	/**
	 * 
	 */
	private static final String CONTENT_0 = "content 0";
	private static final String CONTENT_1 = "content 1";
	private static final String CONTENT_2 = "content 2";

	private static final String ENTRY_0 = "a0/b0/c0";
	private static final String ENTRY_0_WITH_BACK_SLASHES = "a0\\b0\\c0";
	private static final String ENTRY_1 = "a0/b0/c1";
	private static final String ENTRY_2 = "a0/b0/c2";
	private static final String ENTRY_3_SIMILAR_TO_ENTRY_2 = "a0/b0.c2";

	@Test
	public void testHashLength() throws IOException {
		final byte[] result = new MasterHashCalculator().calculateHash(
				"test0.zip", createZipFile0WithContent0());
		assertEquals(20, result.length);
	}

	@Test
	public void testSameFileShouldHaveSameHash() throws IOException {
		final byte[] result0 = new MasterHashCalculator().calculateHash(
				"test0.zip", createZipFile0WithContent0());
		final byte[] result1 = new MasterHashCalculator().calculateHash(
				"test1.zip", createZipFile0WithContent0());
		assertTrue(Arrays.equals(result0, result1));
	}

	@Test
	public void testSameFileByDifferentOperatingSystemsShouldHaveSameHash()
			throws IOException {
		final byte[] result0 = new MasterHashCalculator().calculateHash(
				"test.zip", createZipFile0WithContent0());
		final byte[] result1 = new MasterHashCalculator().calculateHash(
				"test.zip", createZipFile0WithBackSlashesAndContent0());
		assertTrue(Arrays.equals(result0, result1));
	}

	@Test
	public void testOrderShouldNotMatter() throws IOException {
		final byte[] result0 = new MasterHashCalculator().calculateHash(
				"test.zip", createZipFileWithContent1());
		final byte[] result1 = new MasterHashCalculator().calculateHash(
				"test.zip", createZipFileWithContent1ButOtherOrder());
		assertTrue(Arrays.equals(result0, result1));
	}

	@Test
	public void testSameContentSimilarNames() throws IOException {
		final byte[] result0 = new MasterHashCalculator().calculateHash(
				"test.zip", createZipFileWithContent2());
		final byte[] result1 = new MasterHashCalculator().calculateHash(
				"test.zip", createZipFileWithContent2ButOtherName());
		assertFalse(Arrays.equals(result0, result1));
	}

	@Test
	public void testSameNameDifferentContent() throws IOException {
		final byte[] result0 = new MasterHashCalculator().calculateHash(
				"test.zip", createZipFileWithContent3a());
		final byte[] result1 = new MasterHashCalculator().calculateHash(
				"test.zip", createZipFileWithContent3b());
		assertFalse(Arrays.equals(result0, result1));
	}

	private InputStream createZipFile0WithContent0() throws IOException {
		final ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
		final ZipOutputStream zipOutputStream = new ZipOutputStream(
				byteArrayOutputStream);
		createEntry(zipOutputStream, ENTRY_0, CONTENT_0);
		zipOutputStream.close();
		return new ByteArrayInputStream(byteArrayOutputStream.toByteArray());
	}

	private InputStream createZipFile0WithBackSlashesAndContent0()
			throws IOException {
		final ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
		final ZipOutputStream zipOutputStream = new ZipOutputStream(
				byteArrayOutputStream);
		createEntry(zipOutputStream, ENTRY_0_WITH_BACK_SLASHES, CONTENT_0);
		zipOutputStream.close();
		return new ByteArrayInputStream(byteArrayOutputStream.toByteArray());
	}

	private InputStream createZipFileWithContent1() throws IOException {
		final ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
		final ZipOutputStream zipOutputStream = new ZipOutputStream(
				byteArrayOutputStream);
		createEntry(zipOutputStream, ENTRY_0, CONTENT_0);
		createEntry(zipOutputStream, ENTRY_1, CONTENT_1);
		createEntry(zipOutputStream, ENTRY_2, CONTENT_2);
		zipOutputStream.close();
		return new ByteArrayInputStream(byteArrayOutputStream.toByteArray());
	}

	private InputStream createZipFileWithContent1ButOtherOrder()
			throws IOException {
		final ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
		final ZipOutputStream zipOutputStream = new ZipOutputStream(
				byteArrayOutputStream);
		createEntry(zipOutputStream, ENTRY_0, CONTENT_0);
		createEntry(zipOutputStream, ENTRY_2, CONTENT_2);
		createEntry(zipOutputStream, ENTRY_1, CONTENT_1);
		zipOutputStream.close();
		return new ByteArrayInputStream(byteArrayOutputStream.toByteArray());
	}

	private InputStream createZipFileWithContent2() throws IOException {
		final ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
		final ZipOutputStream zipOutputStream = new ZipOutputStream(
				byteArrayOutputStream);
		createEntry(zipOutputStream, ENTRY_2, CONTENT_2);
		zipOutputStream.close();
		return new ByteArrayInputStream(byteArrayOutputStream.toByteArray());
	}

	private InputStream createZipFileWithContent2ButOtherName()
			throws IOException {
		final ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
		final ZipOutputStream zipOutputStream = new ZipOutputStream(
				byteArrayOutputStream);
		createEntry(zipOutputStream, ENTRY_3_SIMILAR_TO_ENTRY_2, CONTENT_2);
		zipOutputStream.close();
		return new ByteArrayInputStream(byteArrayOutputStream.toByteArray());
	}

	private InputStream createZipFileWithContent3a() throws IOException {
		final ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
		final ZipOutputStream zipOutputStream = new ZipOutputStream(
				byteArrayOutputStream);
		createEntry(zipOutputStream, ENTRY_1, CONTENT_1);
		zipOutputStream.close();
		return new ByteArrayInputStream(byteArrayOutputStream.toByteArray());
	}

	private InputStream createZipFileWithContent3b() throws IOException {
		final ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
		final ZipOutputStream zipOutputStream = new ZipOutputStream(
				byteArrayOutputStream);
		createEntry(zipOutputStream, ENTRY_1, CONTENT_2);
		zipOutputStream.close();
		return new ByteArrayInputStream(byteArrayOutputStream.toByteArray());
	}

	private void createEntry(ZipOutputStream zipOutputStream, String name,
			String content) throws IOException {
		zipOutputStream.putNextEntry(new ZipEntry(name));
		zipOutputStream.write(content.getBytes());
		zipOutputStream.closeEntry();
	}

}
