/*
 * Copyright 2007-2009 Florian Köberle
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.delphigl.files.upload.hashcalculation;

import static org.junit.Assert.assertEquals;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.junit.Test;

import com.delphigl.files.hashcalculation.HashCalculationListener;
import com.delphigl.files.hashcalculation.RecursiveHashCalculator;
import com.delphigl.files.hashcalculation.RecursiveHashCalculatorImplementation;

public class RecursiveHashCalculatorTest {

	private final class ProgressRecordingListener implements
			HashCalculationListener {
		private final List<Double> progressRecord = new ArrayList<Double>();

		@Override
		public void handleProgress(double progress) {
			progressRecord.add(progress);
		}

		@Override
		public void hashCalculated(File file, String hexString)
				throws IOException {
			// do nothing
		}

		public List<Double> getProgressRecord() {
			return this.progressRecord;
		}
	}

	private static final int NUMBER_OF_FILES = 5;
	private static final double E = 0.00001;

	@Test
	public void testSimpleCase() throws Exception {
		final ProgressRecordingListener listener = new ProgressRecordingListener();
		File directory = File.createTempFile("testFolderToHash", "");
		directory.delete();
		directory.mkdir();
		directory.deleteOnExit();
		for (int i = 0; i < NUMBER_OF_FILES; i++) {
			File.createTempFile("exampleFile", ".txt", directory);
		}
		final RecursiveHashCalculator recursiveHashCalculator = new RecursiveHashCalculatorImplementation();
		recursiveHashCalculator.calculateHashesOfDirectory(directory, listener);
		final List<Double> actualResult = listener.getProgressRecord();
		assertEquals(NUMBER_OF_FILES, actualResult.size());
		assertEquals(0.2, actualResult.get(0), E);
		assertEquals(0.4, actualResult.get(1), E);
		assertEquals(1.0, actualResult.get(4), E);
		assertEquals(1.0, actualResult.get(NUMBER_OF_FILES - 1), E);
	}

	@Test
	public void testTreeCase() throws Exception {
		final ProgressRecordingListener listener = new ProgressRecordingListener();
		File parentDirectory = File
				.createTempFile("parentTestFolderToHash", "");
		parentDirectory.delete();
		parentDirectory.mkdir();
		parentDirectory.deleteOnExit();
		File.createTempFile("exampleFile", ".txt", parentDirectory);
		final File childDirectory = File.createTempFile("childDirectory",
				".txt", parentDirectory);
		childDirectory.delete();
		childDirectory.mkdir();
		File.createTempFile("exampleFile", ".txt", childDirectory);
		File.createTempFile("exampleFile", ".txt", childDirectory);

		final RecursiveHashCalculator recursiveHashCalculator = new RecursiveHashCalculatorImplementation();
		recursiveHashCalculator.calculateHashesOfDirectory(parentDirectory,
				listener);
		final List<Double> actualResult = listener.getProgressRecord();
		assertEquals(3, actualResult.size());
		assertEquals(0.25, actualResult.get(0), E);
		assertEquals(0.5, actualResult.get(1), E);
		assertEquals(1, actualResult.get(2), E);
	}
}
