/*
 * Copyright 2007-2009 Florian Köberle
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.delphigl.files.uploader.controller;

import static com.delphigl.files.uploader.controller.RemoteUpdateOperation.Type.CREATE;
import static com.delphigl.files.uploader.controller.RemoteUpdateOperation.Type.DELETE;
import static com.delphigl.files.uploader.controller.RemoteUpdateOperation.Type.REPLACE;
import static java.awt.Desktop.getDesktop;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.concurrent.ExecutionException;
import java.util.logging.Formatter;
import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.logging.SimpleFormatter;

import javax.swing.SwingWorker;
import javax.xml.bind.JAXBException;

import com.delphigl.files.ftp.SimpleFTPClient;
import com.delphigl.files.uploader.model.Downloader;
import com.delphigl.files.uploader.model.RemoteUpdateTool;
import com.delphigl.files.uploader.model.SingleLogFileHandler;
import com.delphigl.files.uploader.model.UploadProgressListener;
import com.delphigl.files.uploader.model.Uploader;
import com.delphigl.files.uploader.model.generation.ArchiveGenerationListener;
import com.delphigl.files.uploader.model.generation.ArchiveGenerator;

public class Controller {
	private final Set<ConfigurationPossibilityListener> configurationPossibilityListenerSet = new HashSet<ConfigurationPossibilityListener>();
	private final Set<OperationPossibilityListener> operationPossibilityListenerSet = new HashSet<OperationPossibilityListener>();
	private final Set<ArchiveListGenerationListener> archiveListGenerationListnerSet = new HashSet<ArchiveListGenerationListener>();
	private final Set<RemoteUpdateOperationListener> remoteUpdateOperationListenerSet = new HashSet<RemoteUpdateOperationListener>();
	private final Set<RemoteUpdateProgressListener> remoteUpdateProgressListenerSet = new HashSet<RemoteUpdateProgressListener>();
	private final Set<ConfigurationValidationListener> configurationValidationListenerSet = new HashSet<ConfigurationValidationListener>();
	private final SimpleDateFormat fileNamePrefixDateFormat = new SimpleDateFormat(
			"yyyy-MM-dd_HH-mm_");

	private boolean configuartionSaved;
	private boolean ftpConfigurationValidated;
	private boolean sourceDirectoryValidated;
	private File directoryWithFiles;
	private boolean working;

	private static final String CONFIGURATION_DIRECTORY_NAME = ".dglfileupload";
	private static final String CONFIGURATION_FILE_NAME = "configuration.xml";
	private final Configuration configuration;
	private final File configurationFile;
	private final File logFile;

	private final Logger log;

	public Controller() {
		this.configuartionSaved = true; // has same content like saved file,
		// because it has been loaded from it.
		this.ftpConfigurationValidated = false;
		this.sourceDirectoryValidated = false;
		this.directoryWithFiles = null;
		this.working = false;
		this.log = Logger.getAnonymousLogger();
		try {
			this.logFile = File.createTempFile("log"
					+ fileNamePrefixDateFormat.format(new Date()), ".txt");
			final Formatter formatter = new SimpleFormatter();
			final Handler handler = new SingleLogFileHandler(logFile, formatter);
			log.addHandler(handler);
			handler.setLevel(Level.ALL);
			log.setLevel(Level.ALL);
		} catch (IOException e) {
			log.log(Level.SEVERE, "unable to create log file", e);
			throw new RuntimeException(e);
		}

		final File configurationDirectory = new File(System
				.getProperty("user.home"), CONFIGURATION_DIRECTORY_NAME);
		if (!configurationDirectory.exists()) {
			configurationDirectory.mkdir();
		}
		this.configurationFile = new File(configurationDirectory,
				CONFIGURATION_FILE_NAME);

		Configuration configuration = new Configuration();
		if (configurationFile.exists()) {
			try {
				configuration = Configuration
						.readConfigurationFile(configurationFile);
			} catch (JAXBException e) {
				log.log(Level.WARNING,
						"catched a exception while reading configuration file",
						e);
			}
		}
		this.configuration = configuration;
	}

	public void save() {
		try {
			this.configuration.save(this.configurationFile);
			this.configuartionSaved = true;
			updateUI();
		} catch (JAXBException e) {
			log.log(Level.WARNING,
					"catched a exception while saving configuration file", e);
		} catch (IOException e) {
			log.log(Level.WARNING,
					"catched a exception while saving configuration file", e);
		}
	}

	public void openTargetFolder() {
		try {
			getDesktop().open(directoryWithFiles);
		} catch (IOException e) {
			log
					.log(
							Level.WARNING,
							"catched a exception while opening directory with files",
							e);
		}
	}

	public void openLogFile() {
		try {
			getDesktop().open(logFile);
		} catch (IOException e) {
			log.log(Level.WARNING,
					"catched a exception while opening the log file", e);
		}
	}

	public void validate() {
		handleTaskStart();
		final SwingWorker<Void, Void> worker = new SwingWorker<Void, Void>() {

			@Override
			protected Void doInBackground() throws Exception {
				final SimpleFTPClient simpleFTPClient = createSimpleFTPClient();
				try {
					simpleFTPClient.connect();
				} finally {
					simpleFTPClient.disconnect();
				}
				return null;
			}

			@Override
			protected void done() {
				try {
					sourceDirectoryValidated = new File(getSourceDirectory())
							.isDirectory();
				} catch (Exception e) {
					log
							.log(
									Level.WARNING,
									"catched a exception while validating the source directory",
									e);
				}
				try {
					get();
					ftpConfigurationValidated = true;
				} catch (InterruptedException e) {
					log
							.log(
									Level.WARNING,
									"catched a exception while validating the ftp configuration",
									e);
				} catch (ExecutionException e) {
					log
							.log(
									Level.WARNING,
									"catched a exception while validating the ftp configuration",
									e.getCause());
				}
				handleTaskStop();

				for (ConfigurationValidationListener listener : configurationValidationListenerSet) {
					listener.handleValidationResults();
				}
			}

		};
		worker.execute();
	}

	/**
	 * 
	 */
	public void generate() {
		final File targetDirectory;
		final String dateString = fileNamePrefixDateFormat.format(new Date());
		try {
			targetDirectory = File
					.createTempFile("generated_" + dateString, "");
			targetDirectory.delete();
			targetDirectory.mkdir();
		} catch (IOException e) {
			log
					.log(
							Level.WARNING,
							"catched a exception while generating the directory for the files",
							e);
			return;
		}

		handleTaskStart();
		final ArchiveGenerator archiveGenerator = new ArchiveGenerator(
				new File(getSourceDirectory()), targetDirectory);
		final int numberOfPlannedResults = archiveGenerator
				.getNumberOfPlannedResults();
		for (ArchiveListGenerationListener listener : archiveListGenerationListnerSet) {
			listener.generationStarted(numberOfPlannedResults);
		}
		final SwingWorker<Void, File> worker = new SwingWorker<Void, File>() {
			@Override
			protected Void doInBackground() throws Exception {

				archiveGenerator.addListener(new ArchiveGenerationListener() {
					@Override
					public void archiveGenerated(File file) {
						publish(file);
					}
				});
				archiveGenerator.generate();

				return null;
			}

			@Override
			protected void process(List<File> chunks) {
				for (ArchiveListGenerationListener listener : archiveListGenerationListnerSet) {
					listener.filesGenerated(chunks);
				}
			}

			@Override
			protected void done() {
				try {
					get();
					directoryWithFiles = targetDirectory;
				} catch (InterruptedException e) {
					log.log(Level.WARNING,
							"catched a exception while generating files", e);
				} catch (ExecutionException e) {
					log.log(Level.WARNING,
							"catched a exception while generating files", e
									.getCause());
				}
				handleTaskStop();
			}

		};
		worker.execute();

	}

	/**
	 * @param objects
	 */
	private void handleRemoteUpdateObjects(List<Object> objects) {
		String message = null;
		Double progress = null;
		final List<RemoteUpdateOperation> remoteUpdateOperationsToAdd = new ArrayList<RemoteUpdateOperation>();
		for (Object object : objects) {
			if (object instanceof String) {
				message = (String) object;
			} else if (object instanceof RemoteUpdateOperation) {
				remoteUpdateOperationsToAdd.add((RemoteUpdateOperation) object);
			}
			if (object instanceof Double) {
				progress = (Double) object;
			}
		}
		if (remoteUpdateOperationsToAdd.size() > 0) {
			for (RemoteUpdateOperationListener listener : remoteUpdateOperationListenerSet) {
				listener.newRemoteUpdateOperations(remoteUpdateOperationsToAdd);
			}
		}
		if (message != null) {
			for (RemoteUpdateProgressListener listener : remoteUpdateProgressListenerSet) {
				listener.newMessage(message);
			}
		}
		if (progress != null) {
			for (RemoteUpdateProgressListener listener : remoteUpdateProgressListenerSet) {
				listener.madeProgress(progress);
			}
		}
	}

	public void simulateUpload() {
		handleTaskStart();
		for (RemoteUpdateOperationListener listener : remoteUpdateOperationListenerSet) {
			listener.started();
		}
		final SwingWorker<Void, Object> worker = new SwingWorker<Void, Object>() {
			@Override
			protected Void doInBackground() throws Exception {
				publish(0.0);
				publish("faking upload progress");
				final SimpleFTPClient simpleFTPClient = createSimpleFTPClient();

				final Uploader uploader = new Uploader() {

					@Override
					public void deleteFile(String remotePath)
							throws IOException {
						publish(new RemoteUpdateOperation(DELETE, null,
								remotePath));
					}

					@Override
					public void replaceFile(String remotePath, File localFile)
							throws IOException, FileNotFoundException {
						publish(new RemoteUpdateOperation(REPLACE, localFile,
								remotePath));
					}

					@Override
					public void uploadFile(String remotePath, File localFile)
							throws IOException, FileNotFoundException {
						publish(new RemoteUpdateOperation(CREATE, localFile,
								remotePath));
					}

				};

				final Downloader downloader = new Downloader() {

					@Override
					public String downloadAsString(String remotePath)
							throws IOException {
						return simpleFTPClient.downloadAsUTF8String(remotePath);
					}

				};
				final UploadProgressListener uploadProgressListener = new UploadProgressListener() {
					@Override
					public void handleProgress(double progress) {
						publish(progress);
					}
				};
				simpleFTPClient.connect();
				try {
					RemoteUpdateTool.uploadAll(downloader, uploader,
							directoryWithFiles, uploadProgressListener);
				} finally {
					simpleFTPClient.disconnect();
				}
				publish("fake upload complete");
				return null;
			}

			@Override
			protected void process(List<Object> objects) {
				handleRemoteUpdateObjects(objects);
			}

			@Override
			protected void done() {
				try {
					get();
				} catch (InterruptedException e) {
					log.log(Level.WARNING,
							"catched a exception while simulating upload", e);
				} catch (ExecutionException e) {
					log.log(Level.WARNING,
							"catched a exception while simulating upload", e
									.getCause());
				}
				handleTaskStop();
			}
		};
		worker.execute();
	}

	public void uploadChanges() {
		handleTaskStart();
		for (RemoteUpdateOperationListener listener : remoteUpdateOperationListenerSet) {
			listener.started();
		}
		final SwingWorker<Void, Object> worker = new SwingWorker<Void, Object>() {
			@Override
			protected Void doInBackground() throws Exception {
				final SimpleFTPClient simpleFTPClient = createSimpleFTPClient();

				final Uploader uploader = new Uploader() {

					@Override
					public void deleteFile(String remotePath)
							throws IOException {
						publish("deleting file " + remotePath);
						simpleFTPClient.deleteFile(remotePath);
						publish(new RemoteUpdateOperation(DELETE, null,
								remotePath));
					}

					@Override
					public void replaceFile(String remotePath, File localFile)
							throws IOException, FileNotFoundException {
						publish("uploading replacement of " + remotePath);
						simpleFTPClient.deleteFile(remotePath);
						simpleFTPClient.uploadFile(remotePath, localFile);
						publish(new RemoteUpdateOperation(REPLACE, localFile,
								remotePath));
					}

					@Override
					public void uploadFile(String remotePath, File localFile)
							throws IOException, FileNotFoundException {
						publish("uploading new file " + remotePath);
						simpleFTPClient.uploadFile(remotePath, localFile);
						publish(new RemoteUpdateOperation(CREATE, localFile,
								remotePath));
					}

				};

				final Downloader downloader = new Downloader() {

					@Override
					public String downloadAsString(String remotePath)
							throws IOException {
						publish("downloading hash file");
						return simpleFTPClient.downloadAsUTF8String(remotePath);
					}

				};
				final UploadProgressListener uploadProgressListener = new UploadProgressListener() {
					@Override
					public void handleProgress(double progress) {
						publish(progress);
					}
				};
				simpleFTPClient.connect();
				try {
					RemoteUpdateTool.uploadAll(downloader, uploader,
							directoryWithFiles, uploadProgressListener);
				} finally {
					simpleFTPClient.disconnect();
				}
				publish("upload complete");
				return null;
			}

			@Override
			protected void process(List<Object> objects) {
				handleRemoteUpdateObjects(objects);
			}

			@Override
			protected void done() {
				try {
					get();
				} catch (InterruptedException e) {
					log.log(Level.WARNING,
							"catched a exception while simulating upload", e
									.getCause());
				} catch (ExecutionException e) {
					log.log(Level.WARNING,
							"catched a exception while uploading changes", e
									.getCause());
				}
				handleTaskStop();
			}
		};
		worker.execute();
	}

	private SimpleFTPClient createSimpleFTPClient() throws IOException {
		return new SimpleFTPClient(getFTPHost(), getFTPTargetDirectory(),
				getFTPUsername(), getFTPPassword());
	}

	public String getFTPHost() {
		return this.configuration.getFTPHost();
	}

	public String getFTPPassword() {
		return this.configuration.getFTPPassword();
	}

	public String getFTPTargetDirectory() {
		return this.configuration.getFTPTargetDirectory();
	}

	public String getFTPUsername() {
		return this.configuration.getFTPUsername();
	}

	public String getSourceDirectory() {
		return this.configuration.getSourceDirectory();
	}

	public void setFtp(FTPConfiguration ftp) {
		handleConfigurationChanged();
		this.configuration.setFtp(ftp);
	}

	public void setFTPHost(String host) {
		handleConfigurationChanged();
		this.configuration.setFTPHost(host);
	}

	public void setFTPPassword(String password) {
		handleConfigurationChanged();
		this.configuration.setFTPPassword(password);
	}

	public void setFTPTargetDirectory(String targetDirectory) {
		handleConfigurationChanged();
		this.configuration.setFTPTargetDirectory(targetDirectory);
	}

	public void setFTPUsername(String username) {
		handleConfigurationChanged();
		this.configuration.setFTPUsername(username);
	}

	public void setSourceDirectory(String sourceDirectory) {
		handleConfigurationChanged();
		this.configuration.setSourceDirectory(sourceDirectory);
	}

	private void handleConfigurationChanged() {
		ftpConfigurationValidated = false;
		sourceDirectoryValidated = false;
		configuartionSaved = false;
		directoryWithFiles = null;
		updateUI();
	}

	private void handleTaskStart() {
		working = true;
		updateUI();
	}

	private void handleTaskStop() {
		working = false;
		updateUI();
	}

	public boolean isConfigurationValid() {
		return ftpConfigurationValidated && sourceDirectoryValidated;
	}

	public boolean isConfigurationPossible() {
		return !working;
	}

	public boolean isSaveConfigurationPossible() {
		return !working && isConfigurationValid() && !configuartionSaved;
	}

	public boolean isValidateConfigurationPossible() {
		return !working && !isConfigurationValid();
	}

	public boolean isGenerateArchivesPossible() {
		return !working && isConfigurationValid();
	}

	public boolean isOpenTargetFolderPossible() {
		return directoryWithFiles != null;
	}

	public boolean isSimulateUploadPossible() {
		return !working && isConfigurationValid() && directoryWithFiles != null;
	}

	public boolean isUploadChangesPossible() {
		return !working && isConfigurationValid() && directoryWithFiles != null;
	}

	public boolean isOpenLogFilePossible() {
		return true;
	}

	public boolean isFtpConfigurationValidated() {
		return this.ftpConfigurationValidated;
	}

	public boolean isSourceDirectoryValidated() {
		return this.sourceDirectoryValidated;
	}

	private void updateUI() {
		for (ConfigurationPossibilityListener listener : configurationPossibilityListenerSet) {
			listener
					.handleConfigurationPossibilityChange(isConfigurationPossible());
		}
		for (OperationPossibilityListener listener : operationPossibilityListenerSet) {
			listener.handleOperationPossibilityChange();
		}
	}

	public boolean addConfigurationPossibilityListener(
			ConfigurationPossibilityListener e) {
		return this.configurationPossibilityListenerSet.add(e);
	}

	public boolean addOperationPossibilityListener(
			OperationPossibilityListener e) {
		return this.operationPossibilityListenerSet.add(e);
	}

	public boolean addArchiveListGenerationListener(
			ArchiveListGenerationListener e) {
		return this.archiveListGenerationListnerSet.add(e);
	}

	public boolean addRemoteUpdateOperationListener(
			RemoteUpdateOperationListener e) {
		return this.remoteUpdateOperationListenerSet.add(e);
	}

	public boolean addRemoteUpdateProgressListener(
			RemoteUpdateProgressListener e) {
		return this.remoteUpdateProgressListenerSet.add(e);
	}

	public boolean addConfigurationValidationListener(
			ConfigurationValidationListener e) {
		return this.configurationValidationListenerSet.add(e);
	}

}
