/*
 * Copyright 2007-2009 Florian Köberle
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.delphigl.files.uploader.model;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.io.StringReader;
import java.net.URI;
import java.util.HashMap;
import java.util.Map;

import com.delphigl.files.hashcalculation.HashCalculationListener;
import com.delphigl.files.hashcalculation.RecursiveHashCalculator;
import com.delphigl.files.hashcalculation.RecursiveHashCalculatorImplementation;

public class RemoteUpdateTool {
	private static final String hashFileName = "hash.txt";

	public static void uploadAll(final Downloader downloader,
			final Uploader uploader, final File directoryToUpload,
			final UploadProgressListener listener) throws IOException {
		uploadAll(downloader, uploader, directoryToUpload, listener,
				new RecursiveHashCalculatorImplementation());
	}

	public static void uploadAll(final Downloader downloader,
			final Uploader uploader, final File directoryToUpload,
			final UploadProgressListener listener,
			RecursiveHashCalculator recusiveHashCalculator) throws IOException {
		listener.handleProgress(0);
		final String hashFileAsString = downloader
				.downloadAsString(hashFileName);
		listener.handleProgress(0.05);

		final Map<String, String> remoteNameToHashMap = hashFileToMap(hashFileAsString);
		final Map<String, String> localNameToHashMap = new HashMap<String, String>();
		final HashCalculationListener hashCalculationListener = new HashCalculationListener() {

			@Override
			public void hashCalculated(File file, String hash)
					throws IOException {
				final URI directoryToUploadURI = directoryToUpload.toURI();
				final URI fileURI = file.toURI();
				final URI relativeFileURI = directoryToUploadURI
						.relativize(fileURI);
				final String relativePath = relativeFileURI.toString();

				final String remoteHash = remoteNameToHashMap
						.get(relativeFileURI.toString());
				if (remoteHash == null) {
					uploader.uploadFile(relativePath, file);
				} else {
					if (!remoteHash.equals(hash)) {
						uploader.replaceFile(relativePath, file);
					} else {
						// TODO notify NothingTodoListener
					}
				}
				localNameToHashMap.put(relativePath, hash);
			}

			@Override
			public void handleProgress(double progress) {
				listener.handleProgress(0.05 + 0.9 * progress);
			}

		};
		recusiveHashCalculator.calculateHashesOfDirectory(directoryToUpload,
				hashCalculationListener);
		// deleting files doesn't take much time
		listener.handleProgress(0.90);

		for (String relativeFileURI : remoteNameToHashMap.keySet()) {
			if (!localNameToHashMap.containsKey(relativeFileURI)) {
				uploader.deleteFile(relativeFileURI);
			}
		}
		if (!remoteNameToHashMap.equals(localNameToHashMap)) {
			listener.handleProgress(0.95);
			final File hashFile = File.createTempFile("hashFileToUpload",
					".txt");
			writeHashFile(localNameToHashMap, hashFile);
			uploader.replaceFile(hashFileName, hashFile);
		}
		listener.handleProgress(1);

	}

	private static Map<String, String> hashFileToMap(String hashFileAsString)
			throws IOException {
		final Map<String, String> filesToHashMap = new HashMap<String, String>();
		final BufferedReader checksumFileReader = new BufferedReader(
				new StringReader(hashFileAsString));
		while (true) {
			final String readedLine = checksumFileReader.readLine();
			if (readedLine == null) {
				break;
			}
			final int spaceIndex = readedLine.lastIndexOf(' ');
			if (spaceIndex != -1) {
				final String checksum = readedLine.substring(spaceIndex + 1,
						readedLine.length());
				final String filename = readedLine.substring(0, spaceIndex);
				filesToHashMap.put(filename, checksum);
			}
		}
		return filesToHashMap;
	}

	private static void writeHashFile(
			Map<String, String> generatedFilesToHashMap, File file)
			throws IOException {
		final OutputStream outputStream = new FileOutputStream(file);
		try {
			final PrintWriter printWriter = new PrintWriter(outputStream);
			try {
				for (Map.Entry<String, String> entry : generatedFilesToHashMap
						.entrySet()) {
					printWriter.printf("%s %s%n", entry.getKey(), entry
							.getValue());
				}
			} finally {
				printWriter.close();
			}

		} finally {
			outputStream.close();
		}
	}

}
