/*
 * Copyright 2007-2009 Florian Köberle
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.delphigl.files.uploader.view;

import java.awt.Color;
import java.awt.GridLayout;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import com.delphigl.files.uploader.controller.ConfigurationPossibilityListener;
import com.delphigl.files.uploader.controller.ConfigurationValidationListener;
import com.delphigl.files.uploader.controller.Controller;

class ConfigurationPanel extends JPanel implements
		ConfigurationPossibilityListener {
	private static final long serialVersionUID = 1100891137938826738L;
	private static final Color COLOR_INVALID = new Color(255, 50, 50);
	private final JLabel ftpHostLabel;
	private final JTextField ftpHostTextField;
	private final JLabel ftpUsernameLabel;
	private final JTextField ftpUsernameTextField;
	private final JLabel ftpPasswordLabel;
	private final JPasswordField ftpPasswordTextField;
	private final JLabel ftpTargetDirectoryLabel;
	private final JTextField ftpTargetDirectoryTextField;
	private final JLabel sourceDirectoryLabel;
	private final JTextField sourceDirectoryTextField;

	public ConfigurationPanel(final Controller controller) {
		this.ftpHostTextField = new JTextField(controller.getFTPHost());
		this.ftpHostLabel = new JLabel("ftp host:");
		this.ftpUsernameLabel = new JLabel("ftp username:");
		this.ftpUsernameTextField = new JTextField(controller.getFTPUsername());
		this.ftpPasswordLabel = new JLabel("ftp password:");
		this.ftpPasswordTextField = new JPasswordField(controller
				.getFTPPassword());
		this.ftpTargetDirectoryLabel = new JLabel("ftp target directory:");
		this.ftpTargetDirectoryTextField = new JTextField(controller
				.getFTPTargetDirectory());
		this.sourceDirectoryLabel = new JLabel("source directory:");
		this.sourceDirectoryTextField = new JTextField(controller
				.getSourceDirectory());
		controller.addConfigurationPossibilityListener(this);

		setLayout(new GridLayout(0, 2));// Any number of rows and 2 columns
		add(ftpHostLabel);
		add(ftpHostTextField);
		add(ftpUsernameLabel);
		add(ftpUsernameTextField);
		add(ftpPasswordLabel);
		add(ftpPasswordTextField);
		add(ftpTargetDirectoryLabel);
		add(ftpTargetDirectoryTextField);
		add(sourceDirectoryLabel);
		add(sourceDirectoryTextField);

		ftpTargetDirectoryTextField.setToolTipText("e.g. /a/b");
		ftpHostTextField.setToolTipText("e.g. www.example.org");
		DocumentListener documentListener = new TextFieldChangeListener(
				controller);

		ftpHostTextField.getDocument().addDocumentListener(documentListener);
		ftpUsernameTextField.getDocument()
				.addDocumentListener(documentListener);
		ftpPasswordTextField.getDocument()
				.addDocumentListener(documentListener);
		ftpTargetDirectoryTextField.getDocument().addDocumentListener(
				documentListener);
		sourceDirectoryTextField.getDocument().addDocumentListener(
				documentListener);

		controller
				.addConfigurationValidationListener(new ConfigurationValidationListener() {
					@Override
					public void handleValidationResults() {
						if (controller.isFtpConfigurationValidated()) {
							markAsValid(ftpHostTextField);
							markAsValid(ftpUsernameTextField);
							markAsValid(ftpPasswordTextField);
						} else {
							markAsInvalid(ftpHostTextField);
							markAsInvalid(ftpUsernameTextField);
							markAsInvalid(ftpPasswordTextField);
						}

						if (controller.isSourceDirectoryValidated()) {
							markAsValid(sourceDirectoryTextField);
						} else {
							markAsInvalid(sourceDirectoryTextField);
						}
					}

				});
	}

	private void markAsInvalid(JTextField textField) {
		textField.setBorder(BorderFactory.createLineBorder(COLOR_INVALID, 2));
	}

	private void markAsValid(JTextField textField) {
		textField.setBorder(new JTextField().getBorder());
	}

	private void setEnabledOfAll(boolean enabled) {
		ftpHostTextField.setEnabled(enabled);
		ftpUsernameTextField.setEnabled(enabled);
		ftpPasswordTextField.setEnabled(enabled);
		ftpTargetDirectoryTextField.setEnabled(enabled);
		sourceDirectoryTextField.setEnabled(enabled);
	}

	/**
	 * @author flo
	 * 
	 */
	private final class TextFieldChangeListener implements DocumentListener {
		/**
		 * 
		 */
		private final Controller controller;

		/**
		 * @param model
		 */
		private TextFieldChangeListener(Controller controller) {
			this.controller = controller;
		}

		@Override
		public void changedUpdate(DocumentEvent e) {
			// not used by textfield
		}

		@Override
		public void insertUpdate(DocumentEvent e) {
			storeDataInModel();
		}

		@Override
		public void removeUpdate(DocumentEvent e) {
			storeDataInModel();
		}

		public void storeDataInModel() {
			controller.setFTPHost(ftpHostTextField.getText());
			controller.setFTPUsername(ftpUsernameTextField.getText());
			controller.setFTPPassword(new String(ftpPasswordTextField
					.getPassword()));
			controller.setFTPTargetDirectory(ftpTargetDirectoryTextField
					.getText());
			controller.setSourceDirectory(sourceDirectoryTextField.getText());
		}
	}

	/**
	 * @see com.delphigl.files.uploader.controller.ConfigurationPossibilityListener#handleConfigurationPossibilityChange(boolean)
	 */
	@Override
	public void handleConfigurationPossibilityChange(
			boolean configurationPossible) {
		setEnabledOfAll(configurationPossible);
	}

}