/*
 * Copyright 2007-2009 Florian Köberle
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.delphigl.files.uploader.model;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import org.junit.Test;

import com.delphigl.files.hashcalculation.HashCalculationListener;
import com.delphigl.files.hashcalculation.RecursiveHashCalculator;

public class RemoteUpdateToolTest {
	private static final Object UPLOAD_FILE_METHOD = "upload file method";
	private static final Object REPLACE_FILE_METHOD = "replace file method";
	private static final Object DELETE_FILE_METHOD = "delete file";
	private static final Object DOWNLOAD_AS_STRING_METHOD = "download as String method";
	private static final Object HANDLE_PROGRESS_METHOD = "handle progress method";

	private static final String FILE_A_NAME = "a.zip";
	private static final String FILE_B_NAME = "b.zip";
	private static final String FILE_C_NAME = "c.zip";
	private static final String FILE_D_NAME = "d.zip";

	private static final File FAKE_DIRECTORY = new File("/fake/directory");
	private static final File FILE_B = new File(FAKE_DIRECTORY, FILE_B_NAME);
	private static final File FILE_C = new File(FAKE_DIRECTORY, FILE_C_NAME);
	private static final File FILE_D = new File(FAKE_DIRECTORY, FILE_D_NAME);

	private static final String FILE_A_HASH_OLD = "hashAOld";
	private static final String FILE_B_HASH_OLD = "hashB";
	private static final String FILE_B_HASH_NEW = FILE_B_HASH_OLD;
	private static final String FILE_C_HASH_OLD = "hashCOld";
	private static final String FILE_C_HASH_NEW = "hashCNew";
	private static final String FILE_D_HASH_NEW = "hashDNew";

	private static final String HASH_FILE_NAME = "hash.txt";

	private static final double HASH_PROGRESS_0 = 0.3;
	private static final double HASH_PROGRESS_1 = 0.5;
	private static final double HASH_PROGRESS_2 = 0.7;

	@Test
	public void test1() throws Exception {
		final String oldHashContent = String.format("%s %s\n%s %s\n%s %s",
				FILE_A_NAME, FILE_A_HASH_OLD, FILE_B_NAME, FILE_B_HASH_OLD,
				FILE_C_NAME, FILE_C_HASH_OLD);
		final Recorder recorder = new Recorder(oldHashContent);
		RemoteUpdateTool.uploadAll(recorder, recorder, FAKE_DIRECTORY,
				recorder, new RecursiveHashCalculator() {
					@Override
					public void calculateHashesOfDirectory(File directory,
							HashCalculationListener listener)
							throws IOException {
						assertEquals(FAKE_DIRECTORY, directory);
						listener.hashCalculated(FILE_C, FILE_C_HASH_NEW);
						listener.handleProgress(HASH_PROGRESS_0);
						listener.hashCalculated(FILE_B, FILE_B_HASH_NEW);
						listener.handleProgress(HASH_PROGRESS_1);
						listener.hashCalculated(FILE_D, FILE_D_HASH_NEW);
						listener.handleProgress(HASH_PROGRESS_2);
					}
				});
		final List<List<Object>> records = recorder.getRecords();
		int counter = 0;
		assertProgress(0, records.get(counter));
		counter++;
		assertEquals(Arrays.asList(DOWNLOAD_AS_STRING_METHOD, "hash.txt"),
				records.get(counter));
		counter++;
		assertProgress(0.05, records.get(counter));
		counter++;
		assertEquals(Arrays.asList(REPLACE_FILE_METHOD, FILE_C_NAME, FILE_C),
				records.get(counter));
		counter++;
		assertProgress(0.05 + 0.9 * HASH_PROGRESS_0, records.get(counter));
		counter++;
		assertProgress(0.05 + 0.9 * HASH_PROGRESS_1, records.get(counter));
		counter++;
		assertEquals(Arrays.asList(UPLOAD_FILE_METHOD, FILE_D_NAME, FILE_D),
				records.get(counter));
		counter++;
		assertProgress(0.05 + 0.9 * HASH_PROGRESS_2, records.get(counter));
		counter++;
		assertProgress(0.90, records.get(counter));
		counter++;
		assertEquals(Arrays.asList(DELETE_FILE_METHOD, FILE_A_NAME), records
				.get(counter));
		counter++;
		assertProgress(0.95, records.get(counter));
		counter++;
		assertEquals(3, records.get(counter).size());
		assertEquals(REPLACE_FILE_METHOD, records.get(counter).get(0));
		assertEquals(HASH_FILE_NAME, records.get(counter).get(1));
		counter++;
		assertProgress(1, records.get(counter));

		// lastIndex+1 should be size of records
		counter++;
		assertEquals(counter, records.size());

		final List<String> expectedHashContent = new ArrayList<String>();
		expectedHashContent.add(String.format("%s %s", FILE_B_NAME,
				FILE_B_HASH_NEW));
		expectedHashContent.add(String.format("%s %s", FILE_C_NAME,
				FILE_C_HASH_NEW));
		expectedHashContent.add(String.format("%s %s", FILE_D_NAME,
				FILE_D_HASH_NEW));

		final List<String> actualHashContent = recorder.getNewHashFileContent();
		Collections.sort(actualHashContent);

		assertEquals(expectedHashContent, actualHashContent);
	}

	/**
	 * @author flo
	 * 
	 */
	private final class Recorder implements Uploader, Downloader,
			UploadProgressListener {
		private final List<List<Object>> records = new ArrayList<List<Object>>();
		private final String hashFileContent;
		private List<String> newHashFileContent;

		public Recorder(String hashFileContent) {
			this.hashFileContent = hashFileContent;
			this.newHashFileContent = null;
		}

		@Override
		public void deleteFile(String remotePath) throws IOException {
			records.add(Arrays.<Object> asList(DELETE_FILE_METHOD, remotePath));
		}

		@Override
		public void replaceFile(String remotePath, File localFile)
				throws IOException, FileNotFoundException {
			records.add(Arrays.<Object> asList(REPLACE_FILE_METHOD, remotePath,
					localFile));
			if (remotePath.equals(HASH_FILE_NAME)) {
				this.newHashFileContent = loadLines(localFile);
			}
		}

		@Override
		public void uploadFile(String remotePath, File localFile)
				throws IOException, FileNotFoundException {
			records.add(Arrays.<Object> asList(UPLOAD_FILE_METHOD, remotePath,
					localFile));
		}

		@Override
		public String downloadAsString(String remotePath) throws IOException {
			records.add(Arrays.<Object> asList(DOWNLOAD_AS_STRING_METHOD,
					remotePath));
			return hashFileContent;
		}

		@Override
		public void handleProgress(double progress) {
			records.add(Arrays
					.<Object> asList(HANDLE_PROGRESS_METHOD, progress));
		}

		/**
		 * @return the records
		 */
		public List<List<Object>> getRecords() {
			return this.records;
		}

		/**
		 * @return the newHashFileContent
		 */
		public List<String> getNewHashFileContent() {
			return this.newHashFileContent;
		}

	}

	private static List<String> loadLines(File localFile)
			throws FileNotFoundException, IOException {
		List<String> lines = new ArrayList<String>();
		assertTrue(localFile.exists());
		final FileReader fileReader = new FileReader(localFile);
		try {
			final BufferedReader bufferedReader = new BufferedReader(fileReader);
			while (true) {
				final String line = bufferedReader.readLine();
				if (line == null) {
					break;
				}
				lines.add(line);
			}
		} finally {
			fileReader.close();
		}
		return lines;
	}

	private void assertProgress(double expectedProgress, List<Object> record) {
		assertEquals(2, record.size());
		assertEquals(HANDLE_PROGRESS_METHOD, record.get(0));
		assertEquals(Double.class, record.get(1).getClass());
		assertEquals(expectedProgress, (Double) record.get(1), 0.001);
	}

}
